/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.ships.memory;

import java.util.List;
import java.util.Vector;

import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.engine.Director;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.ships.testing.DataSink;
import com.sunlabs.archsim.fleet.ships.testing.TestingDefaults;
import com.sunlabs.archsim.gasp.State;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.runner.Runner;

import junit.framework.TestCase;

/**
 * @author ib27688
 *
 */
public class SimpleMemoryTest extends TestCase {

	private Vector<Data> readSinkData;
	
	Director director= Director.getInstance();
	
	// used to run tests "manually"
	public static void main(String[] params) {
		junit.textui.TestRunner.run(SimpleMemoryTest.class);
	} // main
	
	public void setUp() {
		
		OperationLoader ldr= new OperationLoader();
		List ops= ldr.load("src/" 
				+ SimpleMemoryTest.class.getPackage().getName().replace('.', '/') 
				+ "/SimpleMemoryTestScript.xml");
		
		Runner r= new Runner();
		r.perform(ops);
		
		readSinkData= ((DataSink)Entities.find("ReadSink")).receivedData;
		
	}
	
	public void tearDown() {
		director.masterClear();
	}
	
	public void setUpManual() {
		// System.out.println("testStride");
		
		Director director= Director.getInstance();
		
		SimpleMemoryTestGroup memoryTest= new SimpleMemoryTestGroup("JUSimpleMemoryTest");
		
		// run self-check
		Boolean selfCheck=  memoryTest.selfCheck();
//		assertTrue(selfCheck);
		
		// run the rest of the test only if self-check passed
		if (selfCheck) {
			director.reset();
			// run for long enough to go through all the tests
			director.run(new Delay(10000));
		}
	}
	
	public void testSelfCheck() {
		
		Object tst=Entities.find("SimpleMemoryTest");
		CompositeEntity testGroup= (CompositeEntity)tst;
		assert(testGroup.selfCheck());
	} // testSelfCheck
	
	public void testReadBeforeWrite() {
		Data d;
		d= readSinkData.elementAt(0);
		assert(d.value.equals(0));
		assertTrue(d.valid);

		d= readSinkData.elementAt(1);
		assert(d.value.equals(0));
		assertTrue(d.valid);
		
		d= readSinkData.elementAt(2);
		assert(d.value.equals(0));
		assertTrue(d.valid);
		
	}
	
	public void testAfterWrite() {
		Data d;

		d= readSinkData.elementAt(3);
		assert(d.value.equals(1));
		assertTrue(d.valid);

		d= readSinkData.elementAt(4);
		assert(d.value.equals(2));
		assertTrue(d.valid);
		
		d= readSinkData.elementAt(5);
		assert(d.value.equals(3));
		assertTrue(d.valid);
		
	}
	
	
} // SimpleMemoryTest

class SimpleMemoryTestGroup  extends CompositeEntity {
	
	SimpleMemoryTestSource source;
	DataSink readSink;
	DataSink writeSink;
	SimpleMemory memory;
	
	public SimpleMemoryTestGroup(String name) {
		super(name);
		
		// make components
		source= new SimpleMemoryTestSource("Source", null);
		addMember(source);
		memory= new SimpleMemory("SimpleMemory");
		addMember(memory);
		readSink= new DataSink("ReadSink");
		addMember(readSink);
		writeSink= new DataSink("WriteSink");
		addMember(writeSink);
		
		// make the states
		State readAddressState= new State("readAddressState", this);
		State ackReadAddressState= new State("ackReadAddressState", this);
		State readDataState= new State("readDataState", this);
		State ackReadDataState= new State("ackReadDataState", this);

		State writeAddressState= new State("writeAddressState", this);
		State ackWriteAddressState= new State("ackWriteAddressState", this);
		State writeDataState= new State("writeDataState", this);
		State ackWriteDataState= new State("ackWriteDataState", this);
		State writeNextState= new State("writeNextState", this);
		State ackWriteNextState= new State("ackWriteNextState", this);

		// make connections
		readAddressState.attach(source.getTerminal(MemoryDefaults.READ_ADDRESS));
		readAddressState.attach(memory.getTerminal(MemoryDefaults.READ_ADDRESS));
		ackReadAddressState.attach(source.getTerminal(MemoryDefaults.ACK_READ_ADDRESS));
		ackReadAddressState.attach(memory.getTerminal(MemoryDefaults.ACK_READ_ADDRESS));
		
		readDataState.attach(memory.getTerminal(MemoryDefaults.READ_DATA));
		readDataState.attach(readSink.getTerminal(TestingDefaults.IN));
		ackReadDataState.attach(memory.getTerminal(MemoryDefaults.ACK_READ_DATA));
		ackReadDataState.attach(readSink.getTerminal(TestingDefaults.ACK_IN));
		
		writeAddressState.attach(source.getTerminal(MemoryDefaults.WRITE_ADDRESS));
		writeAddressState.attach(memory.getTerminal(MemoryDefaults.WRITE_ADDRESS));
		ackWriteAddressState.attach(source.getTerminal(MemoryDefaults.ACK_WRITE_ADDRESS));
		ackWriteAddressState.attach(memory.getTerminal(MemoryDefaults.ACK_WRITE_ADDRESS));
		
		writeDataState.attach(source.getTerminal(MemoryDefaults.WRITE_DATA));
		writeDataState.attach(memory.getTerminal(MemoryDefaults.WRITE_DATA));
		ackWriteDataState.attach(source.getTerminal(MemoryDefaults.ACK_WRITE_DATA));
		ackWriteDataState.attach(memory.getTerminal(MemoryDefaults.ACK_WRITE_DATA));

		writeNextState.attach(memory.getTerminal(MemoryDefaults.WRITE_NEXT));
		writeNextState.attach(writeSink.getTerminal(TestingDefaults.IN));
		ackWriteNextState.attach(memory.getTerminal(MemoryDefaults.ACK_WRITE_NEXT));
		ackWriteNextState.attach(writeSink.getTerminal(TestingDefaults.ACK_IN));
		
		// activate initially active states
		ackReadAddressState.setInitiallyActive(null);
		ackReadDataState.setInitiallyActive(null);
		ackWriteAddressState.setInitiallyActive(null);
		ackWriteDataState.setInitiallyActive(null);
		ackWriteNextState.setInitiallyActive(null);
		
	} // SimpleMemoryTestGroup constructor
	
} // SimpleMemoryTestGroup